//genesis - 3 Neuron tritonia Simulation
prompt		"Tritonia !"		// set the prompt

setclock 0  0.5 	// time step in msec for simulation and spike plot
setclock 1  2.0		// output interval for cell plots
setclock 2 10.0		// output interval for slowly varying graphs (Gk)
int 		GRAPHICS	=	1

//==================================================
//                  STARTUP SCRIPTS
//==================================================
include constants.g
include tools.g
include channel.g
include cell.g
include graphics3.g

function ResetDefaults
    echo "Reset Defaults - not yet implemented"
end

//==================================================
//                      dsi  
//==================================================
makeneuronD(/dsi,30e-4,30e-4)
position /dsi  		100e-4 100e-4 500e-4
set  /dsi/soma          Em  -40    \  //mV
                        Rm  3.0e4  \  //Kohm
                        Cm  4.17e-3 \  //microF
                        inject  -2.0e-4  //microA
                        
//==================================================
//                      c2   
//==================================================
makeneuronC(/c2,30e-4,30e-4)
position /c2   			200e-4 200e-4 500e-4
set /c2/soma            Em  -50      \
                        Rm  1.2e4    \
                        Cm  4.17e-3
//==================================================
//                      vsi(B)
//==================================================
makeneuronV(/vsi,30e-4,30e-4)
position /vsi                   200e-4 100e-4 500e-4
set /vsi/soma           Em  -55      \
                        Rm  1.5e4    \
                        Cm  6.13e-3
//==================================================
//  create a fake neuron ramp for simulation purpose
//==================================================
include makeramp.g

//==================================================
//                  CONNECTIONS
//==================================================
// connections from dsi
connect /dsi/axon       /dsi/soma/dsiNa1          with synapse
connect /dsi/axon       /dsi/soma/dsiNa2          with synapse
connect /dsi/axon       /dsi/soma/dsiK1           with synapse
connect /dsi/axon       /dsi/soma/dsiK2           with synapse
connect /dsi/axon       /c2/soma/dsiNa1           with synapse
connect /dsi/axon       /c2/soma/dsiNa2           with synapse
connect /dsi/axon       /vsi/soma/dsiK            with synapse
// connections from c2
connect /c2/axon        /dsi/soma/c2Na            with synapse
connect /c2/axon        /dsi/soma/c2K             with synapse
connect /c2/axon        /c2/soma/c2K1             with synapse
connect /c2/axon        /c2/soma/c2K2             with synapse
connect /c2/axon        /vsi/soma/c2Na            with synapse
// connections from vsi
connect /vsi/axon       /dsi/soma/vsiK            with synapse
connect /vsi/axon       /c2/soma/vsiK1            with synapse
connect /vsi/axon       /c2/soma/vsiK2            with synapse
connect /vsi/axon       /vsi/soma/vsiNa           with synapse
connect /vsi/axon       /vsi/soma/vsiK1           with synapse
connect /vsi/axon       /vsi/soma/vsiK2           with synapse
// "ramp" input to dsi
connect	/ramp/axon      /dsi/soma/rampNa          with synapse

/* set delay and weight parameters for connections  */
/* initialize to no delay and unit weight for all   */
set	/dsi/axon:#	delay 	0   weight   1  		  
set	/c2/axon:#	delay 	0   weight   1  		  
set	/vsi/axon:#	delay 	0   weight   1  		  
//                              msec         per synapse 
// except ramp gets a higher weight
set     /ramp/axon:0    delay   0   weight   1.5                       

//==================================================
//                    GRAPHICS
//==================================================
if(GRAPHICS)
    loadgraphs
    loadcontrol
end

//==================================================
// check and initialize the simulation
//==================================================
check
reset
adjustspike

echo ==================
echo Simulation loaded.
echo ==================
