//genesis
/* 
	functions defined in this script
=============================================================================
	FUNCTION NAME		ARGUMENTS
=============================================================================
	makeneuronD		(path,soma_l,soma_d)
        makeneuronC             (path,soma_1,soma_d)
        makeneuronV             (path,soma_1,soma_d)
	reset_compt		(comptpath)
	reset_all
============================================================================
*/

//=================================================
//                    NEUROND
//=================================================
function makeneuronD(path,soma_1,soma_d)
    str path
    float soma_1,soma_d

    create      neutral         {path}
    push {path}
    create          compartment      soma
    makechannel(soma,rampNa,{ENa},25.0,15000.0,0.012e-3)
    makechannel(soma,dsiNa1,{ENa},100.0,500.0,0.001e-3)
    makechannel(soma,dsiNa2,{ENa},2000.0,2000.0,0.001e-3)
    makechannel(soma,vsiK,{EK},150.0,150.0,0.6e-3)
    makechannel(soma,c2Na,{ENa},400.0,400.0,0.045e-3)
    makechannel(soma,c2K,{EK},400.0,5000.0,0.0015e-3)
    makechannel(soma,dsiK1,{EK},20.0,20.0,0.025e-3)
    makechannel(soma,dsiK2,{EK},20.0,5000.0,0.00223e-3)

    create      spike           spike
    set         spike   \
                thresh          -48  \
                abs_refract     0   \
                output_amp      1
    create      expthresh       newthresh   
    set         newthresh   \
                tau_theta   50   \
                theta_0     0    \
                theta_s     -48
    sendmsg     newthresh       spike      THRESH state
    sendmsg     spike           newthresh  BUFFER name
    sendmsg     soma            spike      INPUT Vm
    create      axon            axon
    sendmsg     spike           axon       BUFFER name
  pop
end
//=================================================
//                    NEURONC
//=================================================  
function makeneuronC(path,soma_1,soma_d)
    str   path
    float soma_1,soma_d 

    create      neutral         {path}
    push  {path}
    create      compartment     soma    
    makechannel(soma,dsiNa1,{ENa},200.0,800.0,0.02e-3)
    makechannel(soma,dsiNa2,{ENa},200.0,2000.0,0.00001e-3)
    makechannel(soma,vsiK1,{EK},400.0,400.0,0.07e-3)
    makechannel(soma,vsiK2,{EK},400.0,2000.0,0.000008e-3)
    makechannel(soma,c2K1,{EK},40.0,40.0,0.033e-3)
    makechannel(soma,c2K2,{EK},40.0,10000.0,0.00083e-3)
    create      spike             spike
    set         spike   \
                thresh            -30   \
                abs_refract       0   \
                output_amp        1
    create      expthresh         newthresh
    set         newthresh    \
                tau_theta    100  \
                theta_0      -10  \
                theta_s      -30
    sendmsg     newthresh         spike     THRESH state
    sendmsg     spike            newthresh  BUFFER name
    sendmsg     soma              spike     INPUT  Vm
    create      axon              axon
    sendmsg     spike             axon      BUFFER  name
   pop
end
//=================================================
//                    NEURONV
//=================================================
function makeneuronV(path,soma_l,soma_d)
    str path
    float soma_l,soma_d

    create 	neutral		{path}
    push {path}
    create      compartment     soma
    //=============================================
    //          voltage dependent conductances
    //=============================================
    make_hhK(soma,hh_K,{EK})

    //=============================================
    //    synaptic conductances on the dendrite
    //=============================================
    makechannel(soma,vsiNa,{ENa},750.0,3000.0,0.0005e-3)
    makechannel(soma,dsiK,{EK},80.0,800.0,0.003e-3)     
    makechannel(soma,c2Na,{ENa},100.0,1500.0,0.003e-3)
    makechannel(soma,vsiK1,{EK},5.0,5.0,0.08e-3)
    makechannel(soma,vsiK2,{EK},5.0,400.0,0.0075e-3)

    //=============================================
    //                 spike detector
    //=============================================
    create	spike		spike
    set		spike	\
                thresh		-43 \
		abs_refract	0   \
		output_amp	1
    create      expthresh       newthresh
    set         newthresh       \
                tau_theta       10   \
                theta_0         -30  \
                theta_s         -43
    sendmsg     newthresh       spike           THRESH state
    sendmsg     spike           newthresh  BUFFER name
    /* use the soma membrane potential to drive the spike detector */
    sendmsg  	soma 		spike 		INPUT Vm

    //=============================================
    //                 output axon
    //=============================================
    create	axon		axon
    /* use the spike detector output to drive the axon */
    sendmsg	spike		axon		BUFFER name
    pop
end

function reset_compt(comptpath)
    str comptpath
    call {comptpath} RESET
    set {comptpath} Vm {get({comptpath}, {Em})}
    call {comptpath}/##[CLASS=channel]  RESET
end

function reset_all	// I`m not sure that this is really needed!
    reset
    reset_compt(/dsi/soma)
    reset_compt(/c2/soma)
    reset_compt(/vsi/soma)
end
