//genesis
/* 
	functions defined in this script
=============================================================================
	FUNCTION NAME		ARGUMENTS
=============================================================================
	makechannel		(compartment,channel,Ek,tau1,tau2,gsyn)
	make_hhK		(compartment,channel,activearea,EK,Erest)
	gnorm			(tau1,tau2,gsyn)	float
=============================================================================
*/

// different forms for the HH rate coefficients 
int	EXPONENTIAL	=	1
int	SIGMOID		=	2
int	LINOID		=	3

//=================================================
//   normalize gmax to Getting form
//=================================================
function gnorm(tau1,tau2,gsyn)
    float gnorm, gmax
    float tau1,tau2,gsyn
    float taudif = tau2 - tau1
    float tpeak
    float gmmm
    if (taudif == 0.0)
       gmmm = {gsyn / Vex}
    else
       tpeak = {tau2*tau1*log({tau2/tau1})/taudif}
       gmmm  = {gsyn*tau2*(exp({-tpeak/tau2})-exp({-tpeak/tau1}))/taudif}
    end
    return {gmmm}
end
			       
//=================================================
//       BASIC TRANSMITTER ACTIVATED CHANNEL
//=================================================
function makechannel(compartment,channel,Eion,tau1,tau2,gsyn)
    str compartment
    str channel
    float Eion				// mV
    float tau1,tau2			// msec
    float gsyn				// mS
    float gmmm = {gnorm({tau1}, {tau2}, {gsyn})}
    echo {gmmm}
    create	channelC2		{compartment}/{channel}
    set		^ \
		Ek  			{Eion} \	
		tau1			{tau1} \ 
		tau2			{tau2} \ 
		gmax			{gmmm}
    set -env 	{compartment}/{channel} 	gsyn	{gsyn}
     link_channel2 	{compartment}/{channel} 	{compartment}
end

//=========================================================
//                  HH K CHANNEl (exists in /vsi/soma only)
//=========================================================
function make_hhK(compartment,channel,EK)
    str compartment
    str channel
    float EK
    float taum  =  10.0
    float tauh  =  760.0

    create	hh_channel	{compartment}/{channel}
    set		^ \
		Ek 		{EK} \			// mV
 		Gbar		1.287e-3              \	// mmhos(=mS) 
		Xpower		1.0 \
		Ypower		1.0 \   
		X_alpha_FORM	{SIGMOID} \
		X_alpha_A	{1.0/taum}  \           //  1/mV-msec
		X_alpha_B	-5.5  \			// mV
		X_alpha_V0	-40.0            \	// mV
                X_beta_FORM	       {SIGMOID} \
                X_beta_A	{1.0/taum}  \           // 1/msec
  		X_beta_B	5.5         \		// mV
  		X_beta_V0	-40.0       \    	// mV
                Y_alpha_FORM    {SIGMOID}   \
                Y_alpha_A       {1.0/tauh}  \
                Y_alpha_B       6.7         \
                Y_alpha_V0      -68.0       \
                Y_beta_FORM     {SIGMOID}   \
                Y_beta_A        {1.0/tauh}  \
                Y_beta_B        -6.7        \
                Y_beta_V0       -68.0
//           alpha = m/taum, beta = (1-m)/taum for m
//           alpha = h/tauh, beta = (1-h)/tauh for h
 
   link_channel2 	{compartment}/{channel}		{compartment} 
end
