//genesis
// graphics3.g for 3 Neuron Tritonia tutorial

include xtools.g
include helpforms.g

// ==============================================
// toggles the overlaying of graph outputs 
// ==============================================
function overlaytoggle(widget)
    overlay / {get({widget},state)} // from xtools.g
end

// celltoggle is not currently used
// ==============================================
// toggles the display of cell control panels 
// ==============================================
function celltoggle(widget,path)
    if(get({widget},state) == 1)
	xshow {path}
    else
	xhide {path}
    end
end

// =================================================
//   Functions used for popup parameter menus
// =================================================

// Create the form and dialog boxes for display/change of Soma parameters
function make_soma_menu(cell)
    str cell
    str formpath        = "/output/popups/" + cell + "/soma"
    str comppath        = "/" + cell + "/soma"
    create xform {formpath} [438,377,240,215] \
	 -title {cell}" Soma Parameters"
    disable {formpath}
    push {formpath}
    create xbutton  DONE  [0,40,118,30] \
                    -script "set_soma_params "{cell}
    create xbutton  CANCEL  [122,40,118,30] \
                    -script "xhide "{formpath}
    create xdialog  Em    [0,75,240,30]  \
                    -title "Erest (mv)"    -value {get({comppath},Em)}
    create xdialog  Rm    [0,110,240,30] \
                    -title "Rmem (KOhm)"   -value {get({comppath},Rm)}
    create xdialog  Cm    [0,145,240,30] \
                    -title "Cmem (uF)"     -value {get({comppath},Cm)}
    create xdialog  Ra    [0,180,240,30] \
                    -title "Raxial (KOhm)" -value {get({comppath},Ra)}
    pop
end

// The channels for a given cell have names which begin with the name
// of the source cell which connects to them.  For example, the dsi cell
// has a potassium channel, "vsiK", which receives input from an axon of
// the vsi cell.  The function "axonpath(cell,channel)" returns the path
// to this axon from the cell and channel names.
// Thus, {axonpath(dsi,vsiK)} evaluates to "/vsi/axon:@/dsi/soma/vsiK"
function axonpath(cell,channel)
    str axonpath
    str cell, channel, src_abr, src
    str chanpath	= "/" + cell + "/soma/" + channel
    src_abr=substring({channel},0,0)
    if (src_abr == "d")
      src = "dsi"
    else
      if (src_abr == "c")
        src = "c2"
      else
        if (src_abr == "v")
	  src = "vsi"
        else
          src = "ramp" // This must be /ramp/axon for the "rampNa" channel
	end
      end
    end
    return  "/"+{src}+"/axon:@"+{chanpath}    
end

// Create the form and dialog boxes for display/change of channel parameters
function make_channel_menu(cell,channel,chanpath)
    str cell,channel,chanpath
    str formpath	= "/output/popups/" + cell + "/" + channel
    create xform {formpath} [408,377,270,285] \
           -title {cell}" cell: "{channel}" channel"
    disable {formpath}
    push {formpath}
    create xbutton  DONE  [0,40,133,30] \
                    -script "set_ch_params "{cell}" "{channel}
    create xbutton  CANCEL  [135,40,133,30] \
                    -script "xhide "{formpath}
    create xdialog  Ek    [0,75,270,30]    -title "Eequil (mV)" \
		-value {get({chanpath},Ek)}
    create xdialog gmax    [0,110,270,30]  -title "gmax (mS)" \
		-value {get({chanpath},gmax)}
    create xdialog tau1    [0,145,270,30]  -title "Tau 1 (msec)" \
		-value {get({chanpath},tau1)}
    create xdialog tau2    [0,180,270,30]  -title "Tau 2 (msec)" \
		-value {get({chanpath},tau2)}
    create xdialog weight  [0,215,270,30]  -title "weight" \
		-value {get({axonpath({cell},{channel})},weight)}
    create xdialog delay   [0,250,270,30]  -title "delay (msec)" \
		-value {get({axonpath({cell},{channel})},delay)}
    pop
end

// make_cell_menu - Makes menu listing soma and all channels for the cell
// also creates the menus with dialog boxes for soma and channel parameters
function make_cell_menu(cell)
    str cell, chanpath, channel
    str path	= "/output/popups/" + cell
    str cell_form	= "/output/popups/" + cell + "/menu"
    create neutral {path}
    create xform {cell_form} [530,30,150,140] -title {cell}" Parameters"
    push {cell_form}
    create xbutton DONE -script "xhide "{cell_form}
    create xbutton soma -script "xshow_on_top "{path}"/soma"
    make_soma_menu({cell})
    foreach chanpath ({EL(/{cell}/##[TYPE=channelC2])})
        channel = pathname({chanpath}, -tail)
        create xbutton {channel} -script "xshow_on_top "{path}"/"{channel}
        make_channel_menu({cell},{channel},{chanpath})
    end
    pop
    xhide {cell_form}
end

// The following two functions are used by the "DONE" button scripts in
// make_soma_menu and make_channel_menu to make the changes
function set_soma_params(cell)
    str cell
    str formpath	= "/output/popups/" + cell + "/soma"
    str comppath	= "/" + cell + "/soma"
    fieldbutton {comppath} Em {formpath}/Em
    fieldbutton {comppath} Rm {formpath}/Rm
    fieldbutton {comppath} Cm {formpath}/Cm
    fieldbutton {comppath} Ra {formpath}/Ra
    xhide {formpath}
end

function set_ch_params(cell,channel)
    str cell,channel
    str formpath	= "/output/popups/" + cell + "/" + channel
    str chanpath	= "/" + cell + "/soma/" + channel
    fieldbutton {chanpath} Ek {formpath}/Ek
    fieldbutton {chanpath} gmax {formpath}/gmax
    fieldbutton {chanpath} tau1 {formpath}/tau1
    fieldbutton {chanpath} tau2 {formpath}/tau2
    fieldbutton {axonpath({cell},{channel})} weight {formpath}/weight
    fieldbutton {axonpath({cell},{channel})} delay  {formpath}/delay
    xhide {formpath}
end

// functions used in MAIN CONTROL PANEL
//
// change_stepsize  -  sets clock 0 and properly normalizes spike plots
//
function change_stepsize
    float dt  = {get(/output/control/stepsize,value)}
    setclock 0 {dt}
    echo dt = {dt}
    adjustspike
    reset	// added to prevent spurious results
end

// =======================================================
// makegraphics path [x y] - Makes Plots for the Cells
// =======================================================

function makegraphics(cell, fx, fy)
    str cell
    int fx, fy
    str path = "/" + {cell}
    str graphicspath = {path}+"/"+{cell}+"_graphs" //e.g. /dsi/dsi_graphs
    // path gives the location of the cell
    // graphicspath gives the location of the control form for the cell

    // add a toggle for this cell in the main cell control panel
// If used, do later in loadcontrol, AFTER graphs have been created
//    create xtoggle 	/output/cells/{pathname({path},-tail)} \
//			-script "celltoggle <widget> "{graphicspath}

    // =================================================
    //                    main form
    // =================================================
    create xform	{graphicspath}		[{fx},{fy},341,685] \
	-nolabel
    push {graphicspath}
    // =================================================
    //      make graph1 for intracellular potentials
    // =================================================
    create   xgraph     graph1           	[0,0,340,340] \
			-title {cell}" Membrane Potential Vm (mV)" 
//  create   script_out         dograph1
//  set      dograph1           command "graphVm "{path}             
    sendmsg {path}/soma  graph1 PLOT Vm *soma *black
    sendmsg {path}/spike graph1 PLOTSCALE state *spike *red {5*getclock(0)} 0
    set graph1 yoffset 0  YUnits mV
    set graph1 xmin 0 xmax 50000 ymin -80 ymax 20 XUnits msec
    useclock graph1 1
    makegraphscale {graphicspath}/graph1

    // =================================================
    //       make graph2 for synaptic conductances
    // =================================================
    create xgraph 	graph2 	[0,342,340,340] \
			-title "GK (mS)" 
    set graph2 yoffset 0 YUnits "G (mS)"
    set graph2 xmin 0 xmax 50000 ymin -0.1e-4 ymax 5e-4 XUnits msec
    useclock graph2 2     // can use a slow clock for smooth curves
    makegraphscale {graphicspath}/graph2 

    if ({path} == "/dsi")
      sendmsg  {path}/soma/rampNa      	graph2 PLOT Gk *rampNa *black
      sendmsg  {path}/soma/c2Na      	graph2 PLOT Gk *c2Na *blue
      sendmsg  {path}/soma/c2K      	graph2 PLOT Gk *c2K *red
    else
      if ({path} == "/c2")
        sendmsg  {path}/soma/dsiNa1     graph2 PLOT Gk *ds1Na1 *black
        sendmsg  {path}/soma/dsiNa2     graph2 PLOT Gk *ds1Na2 *blue
        sendmsg  {path}/soma/vsiK1      graph2 PLOT Gk *vsiK1  *red
        sendmsg  {path}/soma/vsiK2      graph2 PLOT Gk *vsiK2  *orange
      else
        if ({path} == "/vsi")
          sendmsg  {path}/soma/c2Na    	graph2 PLOT Gk *c2Na *black
          sendmsg  {path}/soma/dsiK    	graph2 PLOT Gk *dsiK *red
          sendmsg  {path}/soma/hh_K    	graph2 PLOT Gk *hh_K *orange
         end
      end
    end
    /* display the widgets and prepare to start */
    xshow {graphicspath}
    set {graphicspath} width 341  // kludge to overcome Xodus bug
    pop
end

// function to create the plot showing spike output of all three cells
function make_spike_plot
    str graphicspath = "/output/spikeplot"
    create xform {graphicspath} [10,390,655,325] -nolabel
    push {graphicspath}
    create   xgraph	graph [3,0,650,320] \
    		-title "Spike Output of the Cells"
    set graph xmin 0 xmax 50000 ymax 6 XUnits msec
    useclock graph 0  // use fast simulation clock to show spikes
    makegraphscale {graphicspath}/graph
    // scale spike plots by stepsize to normalize
    sendmsg /dsi/spike graph PLOTSCALE state *dsi *black {getclock(0)} 4
    sendmsg /vsi/spike graph PLOTSCALE state *vsi *blue {getclock(0)} 2
    sendmsg /c2/spike  graph PLOTSCALE state *c2  *red {getclock(0)} 0
    set {graphicspath} height 325  // kludge to overcome Xodus bug
    pop
end

// =======================================================
//  The two main functions, loadgraphs and loadcontrol
//  set up the forms used for graphs and control buttons
// =======================================================
function loadgraphs
    xon				// activate XODUS
// While forms are being loaded, put up some credits
     echo " "
     echo "*************************************************"
     echo "*                                               *"
     echo "*                  TRITONIA                     *"
     echo "*                                               *"
     echo "*       A simulation of Getting's 3 Neuron      *"
     echo "*           Model of the Tritonia CPG           *"
     echo "*                                               *"
     echo "*               by Jisoon Ihm                   *"
     echo "*     based on a simulation by S. Rykebusch     *"
     echo "*           modified by D. Beeman               *"
     echo "*************************************************"
     echo " "
    xcolorscale hot		// give it color

    // =================================================
    //                     GRAPHICS
    // =================================================
    makegraphics dsi	  0  10
    makegraphics c2     341  10
    makegraphics vsi    682  10
    make_spike_plot

end     // loadgraphs

    // =================================================
    //                MAIN CONTROL PANEL
    // =================================================
function loadcontrol
    create xform 	/output/control [10,720,1000,135] -nolabel
    push /output/control
    create xbutton 	HELP	    [5,2,78,30] \
                        -script "xshow /output/popups/helpmenu"
    create xbutton 	reset 	  [86,2,78,30]   -title RESET \
                        -script reset
    create xbutton 	stop 	  [167,2,78,30]  -title STOP  -script stop
    create xbutton 	quit 	  [248,2,78,30]  -title QUIT \
				    -script "quitbutton <widget>"
    makeconfirm(quit,"Quit Tritonia?","quit","cancelquit <widget>",91,730)
    create xbutton	resetdefaults [5,34,78,30] -title "Reset Deflts" \
			-script "ResetDefaults"
    create xtoggle 	overlay     [86,34,78,30]  -title "" \
				    -script "overlaytoggle <widget>"
    set overlay label0 "Overlay OFF" label1 "Overlay ON" state 0
    overlaytoggle /output/control/overlay
// be sure that it is in state 0 after Reset Defaults
    create xtoggle 	spikeplot     [167,34,159,30]  -title "" \
    			-script "celltoggle <widget> /output/spikeplot"
    set spikeplot label0 "Spike Plot OFF" label1 "Spike Plot ON" state 0
    create xdialog 	step 	    [5,68,240,30] -title "STEP (msec)" \
			-script "stepbutton <widget>"  -value 50000

    create xdialog 	stepsize    [5,102,240,30] -title "dt (msec)" \
			-script "change_stepsize"   -value {getclock(0)}

    create xlabel	cellparms  [835, 2,160,30] -title "Cell Parameters"
    create xbutton	dsiparms  [835,35,160,30] -title "dsi" \
		   -script "xshow_on_top /output/popups/dsi/menu"
    create xbutton	c2parms  [835,68,160,30] -title "c2" \
		   -script "xshow_on_top /output/popups/c2/menu"
    create xbutton	vsiparms  [835,102,160,30] -title "vsi" \
		   -script "xshow_on_top /output/popups/vsi/menu"
    pop
    
// Dialog boxes for injection currents
    create xlabel /output/control/injection [330,2,245,30] \
           -title "Injection Current (uA)"
    push /output/control/injection
    create xdialog dsi_inj [330,35,245,30] -title "dsi Inj" \
	   -script "fieldbutton /dsi/soma inject <widget>" \
	   -value {get(/dsi/soma, inject)}

    create xdialog c2_inj [330,68,245,30] -title "c2 Inj" \
	   -script "fieldbutton /c2/soma inject <widget>" \
	   -value {get(/c2/soma, inject)}

    create xdialog vsi_inj [330,102,245,30] -title "vsi Inj" \
	   -script "fieldbutton /vsi/soma inject <widget>" \
	   -value {get(/vsi/soma, inject)}
    pop

    // =================================================
    //                   Popup Menus
    // =================================================
// Popup Menus for Cell Parameters
    create neutral /output/popups
    make_cell_menu(dsi)
    make_cell_menu(c2)
    make_cell_menu(vsi)

    xshow /output/control
    loadhelp		// set up the help screens and menu (helpforms.g)
    showintro		// show a big GIF image (in helpforms.g)
end			// loadcontrol
